import React from 'react'
import { get, isBoolean, isNumber } from 'lodash'
import { th } from '@pubsweet/ui'
import PropTypes from 'prop-types'
import { connect } from 'react-redux'
import styled from 'styled-components'
import { withRouter } from 'react-router-dom'
import { selectCurrentVersion } from 'xpub-selectors'
import {
  compose,
  withState,
  withProps,
  getContext,
  withHandlers,
  setDisplayName,
} from 'recompose'
import { change as changeForm, formValueSelector } from 'redux-form'
import { SortableList } from 'pubsweet-component-sortable-list/src/components'

import {
  addAuthor,
  deleteAuthor,
  authorFailure,
  getAuthorError,
} from '../../redux/authors'

import { DragHandle } from './FormItems'
import { Author, StaticList, AuthorAdder, AuthorEditor } from './'

const wizardSelector = formValueSelector('wizard')

const Authors = ({
  match,
  error,
  authors,
  version,
  addMode,
  editMode,
  dropItem,
  addAuthor,
  moveAuthor,
  editAuthor,
  setEditMode,
  editedAuthor,
  setFormAuthors,
  ...rest
}) => (
  <Root>
    <AuthorAdder
      addAuthor={addAuthor}
      authors={authors}
      editAuthor={editAuthor}
      editMode={addMode}
      match={match}
      setEditMode={setEditMode}
      setFormAuthors={setFormAuthors}
    />
    {isNumber(editMode) && editMode > -1 ? (
      <StaticList
        authors={authors}
        editComponent={AuthorEditor}
        editIndex={editMode}
        setFormAuthors={setFormAuthors}
        version={version}
        {...rest}
      />
    ) : (
      <SortableList
        beginDragProps={['index', 'lastName']}
        dragHandle={DragHandle}
        editedAuthor={editMode}
        items={authors || []}
        listItem={Author}
        moveItem={moveAuthor}
        {...rest}
      />
    )}
    {error && <ErrorMessage>{error}</ErrorMessage>}
  </Root>
)

export default compose(
  withRouter,
  getContext({ version: PropTypes.object, project: PropTypes.object }),
  connect(
    (state, { project }) => ({
      error: getAuthorError(state),
      currentUser: get(state, 'currentUser.user'),
      version: selectCurrentVersion(state, project),
      authorForm: wizardSelector(state, 'authorForm'),
    }),
    {
      addAuthor,
      changeForm,
      deleteAuthor,
      authorFailure,
    },
  ),
  withState('authors', 'setAuthors', []),
  withProps(({ version, authorForm }) => ({
    authors: get(version, 'authors') || [],
    addMode: isBoolean(authorForm) && authorForm,
    editMode: isNumber(authorForm) ? authorForm : -1,
  })),
  withHandlers({
    setFormAuthors: ({ setAuthors, changeForm }) => (authors = []) => {
      setAuthors(authors)
      changeForm('wizard', 'authors', authors)
    },
  }),
  withHandlers({
    setAuthorEdit: ({ changeForm }) => authorIndex => e => {
      e && e.preventDefault && e.preventDefault()
      changeForm('wizard', 'authorForm', authorIndex)
    },
    setEditMode: ({ changeForm }) => mode => e => {
      e && e.preventDefault()
      changeForm('wizard', 'authorForm', mode)
    },
    parseAuthorType: () => (isSubmitting, isCorresponding, index) => {
      if (isSubmitting) return `#${index + 1} Submitting author`
      if (isCorresponding) return `#${index + 1} Corresponding author`
      return `#${index + 1} Author`
    },
    moveAuthor: ({ authors, setFormAuthors }) => (dragIndex, hoverIndex) => {
      const newAuthors = SortableList.moveItem(authors, dragIndex, hoverIndex)
      setFormAuthors(newAuthors)
    },
    removeAuthor: ({
      authors,
      version,
      project,
      deleteAuthor,
      setFormAuthors,
    }) => id => () => {
      deleteAuthor(project.id, version.id, id).then(() => {
        const newAuthors = authors.filter(a => a.id !== id)
        setFormAuthors(newAuthors)
      })
    },
  }),
  setDisplayName('AuthorList'),
)(Authors)

// #region styled-components
const Root = styled.div`
  border: ${th('borderDefault')};
  padding: ${th('subGridUnit')};
`
const ErrorMessage = styled.div`
  color: ${th('colorError')};
`
// #endregion
