import React from 'react'
import { get } from 'lodash'
import PropTypes from 'prop-types'
import { connect } from 'react-redux'
import { th } from '@pubsweet/ui-toolkit'
import { Button, Icon } from '@pubsweet/ui'
import styled, { css, withTheme } from 'styled-components'
import { compose, getContext, setDisplayName } from 'recompose'
import { DateParser } from 'pubsweet-components-faraday/src/components'
import { AuthorsWithTooltip } from 'pubsweet-component-manuscript/src/molecules'

import ZipFiles from '../Files/ZipFiles'
import { Decision, Recommendation } from '../'
import { InviteReviewers } from '../Reviewers/'
import { selectInvitation } from '../../redux/reviewers'
import { ReviewerDecision, HandlingEditorSection, DeleteManuscript } from './'
import { parseVersion, parseJournalIssue, mapStatusToLabel } from './../utils'
import {
  canMakeDecision,
  isHEToManuscript,
  canInviteReviewers,
  canMakeRecommendation,
} from '../../../../component-faraday-selectors/src'

import { ReviewerBreakdown } from '../Invitations'

const DashboardCard = ({
  isHE,
  theme,
  journal,
  history,
  project,
  version,
  invitation,
  currentUser,
  deleteProject,
  canMakeDecision,
  canInviteReviewers,
  canMakeRecommendation,
  ...rest
}) => {
  const files = get(version, 'files')
  const metadata = get(version, 'metadata')
  const { title, type } = parseVersion(version)
  const customId = project.customId || project.id.split('-')[0]
  const hasFiles = files ? Object.values(files).some(f => f.length > 0) : false
  const journalIssueType = parseJournalIssue(journal, metadata)
  const submittedDate = get(version, 'submitted')
  const manuscriptMeta = `${type} - ${
    journalIssueType ? journalIssueType.label : 'N/A'
  }`

  return version ? (
    <Card id={customId}>
      <ListView>
        <Top>
          <LeftDetails flex={6}>
            <ManuscriptId>{`ID ${customId}`}</ManuscriptId>
            <Title
              title={title}
              dangerouslySetInnerHTML={{ __html: title }} // eslint-disable-line
            />
            <RightDetails flex="none">
              {canMakeDecision && (
                <Decision
                  collectionId={project.id}
                  fragmentId={version.id}
                  modalKey={`decide-${version.id}`}
                />
              )}
              {canMakeRecommendation && (
                <Recommendation
                  collectionId={project.id}
                  fragmentId={version.id}
                  modalKey={`recommend-${version.id}`}
                />
              )}
              <ZipFiles
                archiveName={`ID-${project.customId}`}
                collectionId={project.id}
                disabled={!hasFiles}
                fragmentId={version.id}
              >
                <ClickableIcon disabled={!hasFiles}>
                  <Icon>download</Icon>
                </ClickableIcon>
              </ZipFiles>
              {(!project.status ||
                project.status === 'draft' ||
                !submittedDate) && (
                <ActionButtons
                  data-test="button-resume-submission"
                  onClick={() =>
                    history.push(
                      `/projects/${project.id}/versions/${version.id}/submit`,
                    )
                  }
                >
                  RESUME SUBMISSION
                </ActionButtons>
              )}
            </RightDetails>
          </LeftDetails>
        </Top>
        <Bottom>
          <LeftDetails flex={3}>
            <Status>{mapStatusToLabel(project)}</Status>
            {submittedDate && (
              <DateParser durationThreshold={0} timestamp={submittedDate}>
                {(timestamp, daysAgo) => (
                  <DateField>
                    {timestamp} ({daysAgo} ago)
                  </DateField>
                )}
              </DateParser>
            )}
          </LeftDetails>
          <RightDetails flex={4}>
            <ManuscriptType title={manuscriptMeta}>
              {manuscriptMeta}
            </ManuscriptType>
            {project.status && project.status !== 'draft' ? (
              submittedDate && (
                <Details
                  data-test="button-details"
                  onClick={() =>
                    history.push(
                      `/projects/${project.id}/versions/${version.id}/details`,
                    )
                  }
                >
                  Details
                  <Icon primary>chevron-right</Icon>
                </Details>
              )
            ) : (
              <DeleteManuscript
                deleteProject={() => deleteProject(project)}
                modalKey={`delete-${version.id}`}
              />
            )}
          </RightDetails>
        </Bottom>
      </ListView>
      {project.status &&
        project.status !== 'draft' && (
          <DetailsView>
            <Top>
              <AuthorsWithTooltip authors={version.authors} />
            </Top>
            <Bottom>
              <LeftDetails flex={4}>
                <HandlingEditorSection
                  currentUser={currentUser}
                  isHE={isHE}
                  project={project}
                />
              </LeftDetails>
              {canInviteReviewers && (
                <RightDetails flex={4}>
                  <ReviewerBreakdown
                    collectionId={project.id}
                    compact
                    versionId={version.id}
                  />
                  <InviteReviewers
                    modalKey={`invite-reviewers-${project.id}`}
                    project={project}
                    version={version}
                  />
                </RightDetails>
              )}
              {invitation && (
                <RightDetails flex={4}>
                  <ReviewerText>Invited to review</ReviewerText>
                  <ReviewerDecision
                    invitation={invitation}
                    modalKey={`reviewer-decision-${project.id}`}
                    project={project}
                    version={version}
                  />
                </RightDetails>
              )}
            </Bottom>
          </DetailsView>
        )}
    </Card>
  ) : null
}

export default compose(
  setDisplayName('DashboardCard'),
  getContext({ journal: PropTypes.object, currentUser: PropTypes.object }),
  withTheme,
  connect((state, { project, version }) => ({
    canMakeDecision: canMakeDecision(state, project),
    isHE: isHEToManuscript(state, get(project, 'id')),
    canInviteReviewers: canInviteReviewers(state, project),
    invitation: selectInvitation(state, get(version, 'id')),
    canMakeRecommendation: canMakeRecommendation(state, project),
  })),
)(DashboardCard)

// #region styled-components
const defaultText = css`
  color: ${th('colorText')};
  font-family: ${th('fontReading')};
  font-size: ${th('fontSizeBaseSmall')};
`

const ReviewerText = styled.div`
  ${defaultText};
  margin-right: ${th('subGridUnit')};
  text-transform: uppercase;
`

const ActionButtons = styled(Button)`
  ${defaultText};
  align-items: center;
  background-color: ${th('colorPrimary')};
  color: ${th('colorTextReverse')};
  display: flex;
  height: calc(${th('subGridUnit')} * 5);
  padding: calc(${th('subGridUnit')} / 2) ${th('subGridUnit')};
  text-align: center;
  white-space: nowrap;
`

const LeftDetails = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;
  justify-content: flex-start;
  flex: ${({ flex }) => flex || 1};
  flex-wrap: wrap;
  max-width: 100%;
`

const RightDetails = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;
  justify-content: flex-end;
  flex: ${({ flex }) => flex || 1};
  flex-wrap: wrap;
`

const DetailsView = styled.div`
  align-items: center;
  border-top: ${th('borderDefault')};
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  width: 100%;
`

const ListView = styled.div`
  align-items: center;
  display: flex;
  flex-direction: column;
  justify-content: space-between;
  width: 100%;
`

const ManuscriptId = styled.div`
  ${defaultText};
  margin-right: ${th('subGridUnit')};
  text-align: left;
  text-transform: uppercase;
  white-space: nowrap;
  flex: 1;
`

const Details = styled.div`
  align-items: center;
  cursor: pointer;
  display: flex;
  ${defaultText};
  margin-left: calc(${th('subGridUnit')} * 2);
  text-decoration: underline;
  text-align: center;
  text-transform: uppercase;
  white-space: nowrap;
`

const ClickableIcon = styled.div`
  cursor: ${({ disabled }) => (disabled ? 'not-allowed' : 'pointer')};
  margin: 0 ${th('subGridUnit')};

  svg {
    stroke: ${({ disabled, theme }) =>
      disabled ? theme.colorBackgroundHue : theme.colorPrimary};
  }
`

const Card = styled.div`
  align-items: center;
  border: ${th('borderDefault')};
  display: flex;
  flex-direction: column;
  justify-content: flex-start;
  margin-bottom: calc(${th('subGridUnit')} * 2);
  background-color: ${th('backgroundColorReverse')};
`

const Top = styled.div`
  display: flex;
  flex-direction: row;
  align-items: center;
  padding: calc(${th('subGridUnit')} * 2);
  width: 100%;
  box-sizing: border-box;
  flex-wrap: wrap;
`
const Bottom = styled.div`
  box-sizing: border-box;
  display: flex;
  flex-direction: row;
  flex-wrap: wrap;
  justify-content: space-between;
  padding: calc(${th('subGridUnit')} * 2);
  padding-top: 0;
  width: 100%;
`

const ManuscriptType = styled.div`
  ${defaultText};
  flex: 1;
  margin-left: calc(${th('subGridUnit')} * 2);
  padding: ${th('subGridUnit')} calc(${th('subGridUnit')} / 2);
  text-align: right;
  text-transform: capitalize;
  text-overflow: ellipsis;
  overflow: hidden;
  white-space: nowrap;
`

const Title = styled.div`
  ${defaultText};
  font-family: ${th('fontHeading')};
  color: ${th('colorPrimary')};
  font-weight: bold;
  text-align: left;
  text-overflow: ellipsis;
  white-space: nowrap;
  overflow: hidden;
  flex: 10;
  padding-right: ${th('subGridUnit')};
`

const Status = styled.div`
  border: ${th('borderDefault')};
  ${defaultText};
  font-weight: bold;
  padding: 0 0.5em;
  text-align: left;
  text-transform: capitalize;
  line-height: 1.5;
  color: ${th('colorPrimary')};
`

const DateField = styled.span`
  ${defaultText};
  margin: 0 ${th('subGridUnit')};
  text-align: left;
`
// #endregion
