import React, { Fragment } from 'react'
import { capitalize } from 'lodash'
import { connect } from 'react-redux'
import { required } from 'xpub-validators'
import styled, { css } from 'styled-components'
import { reduxForm, change as changeForm } from 'redux-form'
import { compose, withState, withHandlers } from 'recompose'
import {
  th,
  Icon,
  Button,
  Spinner,
  RadioGroup,
  ValidatedField,
} from '@pubsweet/ui'

import { utils } from './'
import { FormItems } from '../UIComponents'

const {
  Row,
  Err,
  Label,
  Title,
  RowItem,
  TextAreaField,
  RootContainer,
  FormContainer,
  CustomRadioGroup,
} = FormItems

const Form = RootContainer.withComponent(FormContainer)

const StepTwo = ({
  goBack,
  hasNote,
  decision,
  showNote,
  removeNote,
  isFetching,
  handleSubmit,
  recommendationError,
}) => (
  <Form onSubmit={handleSubmit}>
    <Title>
      {decision !== 'revise'
        ? `Recommandation to ${capitalize(decision)}`
        : `Request a revision from Author`}
    </Title>
    {decision !== 'revise' ? (
      <Fragment>
        <Row>
          <RowItem vertical>
            <Label>Message for Editor in Chief (optional)</Label>
            <ValidatedField component={TextAreaField} name="message.eic" />
          </RowItem>
        </Row>
        <Row>
          <RowItem vertical>
            <Label>Message for Author (optional)</Label>
            <ValidatedField component={TextAreaField} name="message.author" />
          </RowItem>
        </Row>
        {recommendationError && (
          <Row>
            <RowItem centered>
              <Err>{recommendationError}</Err>
            </RowItem>
          </Row>
        )}
      </Fragment>
    ) : (
      <Fragment>
        <Row>
          <RowItem vertical>
            <Label>REVISION TYPE</Label>
            <ValidatedField
              component={input => (
                <CustomRadioGroup justify="flex-start">
                  <RadioGroup
                    name="revision.revision-type"
                    {...input}
                    options={utils.revisionOptions}
                  />
                </CustomRadioGroup>
              )}
              name="revision.revisionType"
              validate={[required]}
            />
          </RowItem>
        </Row>
        <Row>
          <RowItem vertical>
            <Label>
              REASON & DETAILS
              <SubLabel>Required</SubLabel>
            </Label>
            <ValidatedField
              component={TextAreaField}
              name="revision.reason"
              validate={[required]}
            />
          </RowItem>
        </Row>
        {!hasNote ? (
          <Row>
            <RowItem>
              <TextButton onClick={showNote}>Add Internal Note</TextButton>
              <HintText>Not shared with author</HintText>
            </RowItem>
          </Row>
        ) : (
          <Fragment>
            <Row noMargin>
              <RowItem flex={2}>
                <Label>
                  INTERNAL NOTE
                  <SubLabel>Not shared with author</SubLabel>
                </Label>
              </RowItem>
              <CustomRowItem onClick={removeNote}>
                <IconButton>
                  <Icon primary>x</Icon>
                </IconButton>
                <TextButton>Remove</TextButton>
              </CustomRowItem>
            </Row>
            <Row noMargin>
              <RowItem>
                <ValidatedField
                  component={TextAreaField}
                  name="revision.internal-note"
                />
              </RowItem>
            </Row>
          </Fragment>
        )}
      </Fragment>
    )}
    <Row>
      <RowItem centered>
        <Button onClick={goBack}>Back</Button>
      </RowItem>
      <RowItem centered>
        {isFetching ? (
          <Spinner size={3} />
        ) : (
          <Button primary type="submit">
            Submit
          </Button>
        )}
      </RowItem>
    </Row>
  </Form>
)

export default compose(
  connect(null, { changeForm }),
  withState('hasNote', 'changeHasNote', false),
  withHandlers({
    showNote: ({ changeHasNote }) => () => {
      changeHasNote(true)
    },
    removeNote: ({ changeHasNote, changeForm }) => () => {
      changeHasNote(false)
      changeForm('recommendation', 'revision.internal-note', '')
    },
  }),
  reduxForm({
    form: 'recommendation',
    destroyOnUnmount: false,
    forceUnregisterOnUnmount: true,
  }),
)(StepTwo)

// #region styled components
const defaultText = css`
  color: ${th('colorPrimary')};
  font-family: ${th('fontInterface')};
  font-size: ${th('fontSizeBaseSmall')};
`

const SubLabel = styled.span`
  ${defaultText};
  font-style: oblique;
  margin-left: ${th('subGridUnit')};
  text-transform: capitalize;
`

const TextButton = styled.span`
  ${defaultText};
  cursor: pointer;
  margin-right: ${th('subGridUnit')};
  text-decoration: underline;
`

const HintText = styled.span`
  ${defaultText};
  font-style: oblique;
`

const IconButton = styled.div`
  display: flex;
  justify-content: center;
`

const CustomRowItem = RowItem.extend`
  align-items: center;
  justify-content: flex-end;

  & > div {
    justify-content: flex-end;
  }
`
// #endregion
