import { omit } from 'lodash'
import { withJournal } from 'xpub-journal'
import { create } from 'pubsweet-client/src/helpers/api'
import { loginUser } from 'pubsweet-component-login/actions'
import { compose, withState, withProps, withHandlers } from 'recompose'

import SignUpInvitation from './SignUpInvitationForm'
import { parseSignupAuthor, handleFormError } from '../utils'

const login = (dispatch, values, history) =>
  dispatch(loginUser(values))
    .then(() => {
      history.push('/')
    })
    .catch(handleFormError)

const confirmUser = (email, token, history) => (values, dispatch) => {
  const request = { ...values, email, token }
  if (values) {
    return create('/users/reset-password', omit(request, ['confirmPassword']))
      .then(r => {
        const { username } = r
        const { password } = values
        login(dispatch, { username, password }, history)
      })
      .catch(handleFormError)
  }
}

const signUpUser = history => (values, dispatch) =>
  create('/users', parseSignupAuthor(values))
    .then(r => {
      const { username } = r
      const { password } = values
      login(dispatch, { username, password }, history).then(() => {
        create('/emails', {
          email: values.email,
          type: 'signup',
        })
      })
    })
    .catch(handleFormError)

const resetUserPassword = history => ({ email }, dispatch) =>
  create(`/users/forgot-password`, { email })
    .then(r => {
      // go to dedicated route
      history.push('/')
    })
    .catch(handleFormError)

const setNewPassword = history => ({ email, token, password }, dispatch) =>
  create(`/users/reset-password`, { email, token, password })
    .then(() => {
      login(dispatch, { username: email, password }, history).then(() =>
        history.push('/'),
      )
    })
    .catch(handleFormError)

export default compose(
  withJournal,
  withState(
    'step',
    'changeStep',
    ({ type }) => (type === 'forgotPassword' || type === 'setPassword' ? 1 : 0),
  ),
  withProps(({ location }) => {
    const params = new URLSearchParams(location.search)
    const email = params.get('email') || ''
    const token = params.get('token') || ''
    const title = params.get('title') || ''
    const lastName = params.get('lastName') || ''
    const firstName = params.get('firstName') || ''
    const affiliation = params.get('affiliation') || ''

    return {
      initialValues: {
        email,
        title,
        token,
        lastName,
        firstName,
        affiliation,
      },
    }
  }),
  withHandlers({
    nextStep: ({ changeStep }) => () => changeStep(step => step + 1),
    prevStep: ({ changeStep }) => () => changeStep(step => step - 1),
    confirmInvitation: ({
      initialValues: { email = '', token = '' },
      history,
    }) => confirmUser(email, token, history),
    signUp: ({ history }) => signUpUser(history),
    forgotPassword: ({ history }) => resetUserPassword(history),
    setNewPassword: ({ history }) => setNewPassword(history),
  }),
  withProps(
    ({ type, signUp, confirmInvitation, forgotPassword, setNewPassword }) => {
      switch (type) {
        case 'forgotPassword':
          return { onSubmit: forgotPassword }
        case 'signup':
          return { onSubmit: signUp }
        case 'setPassword':
          return { onSubmit: setNewPassword }
        default:
          return { onSubmit: confirmInvitation }
      }
    },
  ),
)(SignUpInvitation)
